package com.chelvc.cloud.vehicle.server.service.impl;

import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.vehicle.api.dto.MerchantAuthDTO;
import com.chelvc.cloud.vehicle.api.dto.NoticeDTO;
import com.chelvc.cloud.vehicle.api.param.MerchantAuthModifyParam;
import com.chelvc.cloud.vehicle.api.param.MerchantAuthPagingParam;
import com.chelvc.cloud.vehicle.api.param.NoticeModifyParam;
import com.chelvc.cloud.vehicle.api.param.NoticePagingParam;
import com.chelvc.cloud.vehicle.server.copier.MerchantAuthCopier;
import com.chelvc.cloud.vehicle.server.copier.NoticeCopier;
import com.chelvc.cloud.vehicle.server.dao.MerchantAuthMapper;
import com.chelvc.cloud.vehicle.server.dao.NoticeMapper;
import com.chelvc.cloud.vehicle.server.entity.MerchantAuth;
import com.chelvc.cloud.vehicle.server.entity.Notice;
import com.chelvc.cloud.vehicle.server.service.MerchantAuthService;
import com.chelvc.cloud.vehicle.server.service.NoticeService;
import com.chelvc.framework.base.context.SessionContextHolder;
import com.chelvc.framework.base.util.ResourceUtils;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.util.StringUtils;
import com.chelvc.framework.database.util.PagingUtils;
import lombok.NonNull;
import lombok.RequiredArgsConstructor;
import org.apache.dubbo.config.annotation.DubboService;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.Date;

/**
 * 商家认证业务操作实现
 *
 * @author liude
 * @date 2023/12/2
 */
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
@DubboService(interfaceClass = com.chelvc.cloud.vehicle.api.service.MerchantAuthService.class)
public class MerchantAuthServiceImpl extends ServiceImpl<MerchantAuthMapper, MerchantAuth> implements MerchantAuthService,
        com.chelvc.cloud.vehicle.api.service.MerchantAuthService {
    @Override
    public Long addMerchantAuth(@NonNull MerchantAuthModifyParam param){
        MerchantAuth merchantAuth = MerchantAuthCopier.INSTANCE.copying(param);
        Long userId = SessionContextHolder.getId();
        merchantAuth.setApplicationTime(new Date());
        merchantAuth.setApplicant(userId);
        this.save(merchantAuth);
        return merchantAuth.getId();
    }

    @Override
    public void updateMerchantAuth(@NonNull Long id, @NonNull MerchantAuthModifyParam param) {
        MerchantAuth merchantAuth = ResourceUtils.required(this.getById(id), "商家认证不存在");
        MerchantAuthCopier.INSTANCE.copying(param, merchantAuth);
        this.updateById(merchantAuth);
    }

    @Override
    public Pagination<MerchantAuthDTO> getMerchantAuthPaging(@NonNull MerchantAuthPagingParam param) {
        Page<MerchantAuth> page = this.lambdaQuery()
                .like(StringUtils.nonEmpty(param.getStoreName()), MerchantAuth::getStoreName, param.getStoreName())
                .orderByAsc(MerchantAuth::getReviewTime).page(PagingUtils.convert(param.getPaging()));
        return PagingUtils.convert(page, MerchantAuthCopier.INSTANCE::copying);
    }
    @Override
   public MerchantAuthDTO queryById(){
        Long userId = SessionContextHolder.getId();
        MerchantAuth merchantAuth = this.lambdaQuery()
                .eq(MerchantAuth::getApplicant, userId).one();
        return MerchantAuthCopier.INSTANCE.copying(merchantAuth);
    }

    @Override
    public void certificationAudit(Long id,String state){
        Long userId = SessionContextHolder.getId();
        MerchantAuth merchantAuth = ResourceUtils.required(this.getById(id), "商家认证不存在");
        merchantAuth.setReviewStatus(state);
        merchantAuth.setReviewTime(new Date());
        merchantAuth.setReviewer(userId);
        this.updateById(merchantAuth);
    }
}
