package com.chelvc.cloud.vehicle.server.service.impl;

import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.LambdaUpdateWrapper;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.core.toolkit.Wrappers;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.chelvc.cloud.vehicle.api.dto.DynamicContentDTO;
import com.chelvc.cloud.vehicle.api.dto.OmsOrderDTO;
import com.chelvc.cloud.vehicle.api.dto.OmsOrderPayHistoryDTO;
import com.chelvc.cloud.vehicle.server.dao.OmsOrderPayHistoryMapper;
import com.chelvc.cloud.vehicle.server.entity.DynamicContent;
import com.chelvc.cloud.vehicle.server.entity.OmsOrder;
import com.chelvc.cloud.vehicle.server.entity.OmsOrderOperateHistory;
import com.chelvc.cloud.vehicle.server.entity.OmsOrderPayHistory;
import com.chelvc.cloud.vehicle.server.service.IOmsOrderService;
import com.chelvc.cloud.vehicle.server.service.OmsOrderPayHistoryService;
import com.chelvc.framework.common.model.Pagination;
import com.chelvc.framework.common.model.Paging;
import com.chelvc.framework.database.util.PagingUtils;
import lombok.RequiredArgsConstructor;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.stream.Collectors;

@Service
@RequiredArgsConstructor(onConstructor = @__(@Autowired))
public class OmsOrderPayHistoryServiceImpl extends ServiceImpl<OmsOrderPayHistoryMapper, OmsOrderPayHistory> implements OmsOrderPayHistoryService {

    private final IOmsOrderService omsOrderService;

    @Override
    public Integer queryNewInformNum(Long userId) {
        LambdaQueryWrapper<OmsOrderPayHistory> lwq = Wrappers.lambdaQuery();
        lwq.eq(OmsOrderPayHistory::getUserId, userId);
        lwq.eq(OmsOrderPayHistory::getFlag, 0);
        return baseMapper.selectCount(lwq);
    }

    @Override
    public void changeUserPayInformFlag(Long userId) {
        LambdaUpdateWrapper<OmsOrderPayHistory> lwq = Wrappers.lambdaUpdate();
        lwq.eq(OmsOrderPayHistory::getUserId, userId);
        lwq.eq(OmsOrderPayHistory::getFlag, 0);
        lwq.set(OmsOrderPayHistory::getFlag, 1);
        baseMapper.update(null, lwq);
    }

    @Override
    public Pagination<OmsOrderDTO> selectOmsOrderPayHistoryListByUserId(Long userId, Paging paging) {
        Page<OmsOrderDTO> page = new Page<>(paging.getIndex(), paging.getSize());
        IPage<OmsOrderDTO> result = baseMapper.selectOmsOrderPayHistoryListByUserId(page, queryWrapper(userId));
        return Pagination.<OmsOrderDTO>builder().total(result.getTotal()).pages(result.getPages())
                .records(result.getRecords()).build();
    }

    private QueryWrapper<OmsOrderPayHistory> queryWrapper(Long userId) {
        QueryWrapper<OmsOrderPayHistory> lqw = Wrappers.query();
        lqw.eq("t1.user_id", userId);
        lqw.orderByDesc("t1.create_time","t1.id");
        return lqw;
    }
}
